(** Managing objects: source, intrinsic/extrinsic description, evolution. *)

module type PARAM =
  sig
  end

module type T =
    sig
      module CRoot : Logcache.ROOT
      module Root : Root.T with module CRoot = CRoot
      module Src : Source.T

(*
      module Update :
	  sig
	    type t = Add of CRoot.t | Sub of CRoot.t | Seq of t * t | Nop

	    val of_query : Root.Query.t -> t
	    val of_descr : CRoot.t list -> t
	    val to_descr : t -> CRoot.t list
	    val parse : Syntax.t_stream -> t
	    val print : t -> Syntax.t_list
	  end
*)

      module Descr :
	  sig
	    type t = CRoot.t list

	    val empty : t
	    val entails : t -> CRoot.t -> bool
	    val update : t -> Root.Update.t -> t * Root.Update.t
	    val of_query : Root.Query.t -> t
	    val parse : Syntax.t_stream -> t
	    val print : t -> Syntax.t_list
	  end

      type origin = External of Src.t * Src.ref | Internal of Source.preview * Source.action_arg list

      val get_oid : Src.t * Src.ref -> Logcache.oid
      val get_origin : Logcache.oid -> origin
      val get_extr : Logcache.oid -> Descr.t
      val get_descr : Logcache.oid -> Descr.t

      val make : origin * Descr.t * Descr.t -> Source.update list
      val update : Logcache.oid * Descr.t -> Source.update list
      val move : Logcache.oid -> Root.Update.t -> Source.update list
      val remove : Logcache.oid -> Source.update list
      val move_intr : Logcache.oid -> Root.Update.t -> Source.update list

      type data

      val init : unit -> unit
      val save : unit -> data
      val load : data -> unit

    end 

module Make (Param : PARAM) (CFeat : Logcache.T) (Src : Source.T) : T =
  struct
    module RootParam = struct end
    module Root = Root.Make(RootParam)(CFeat)
    module CRoot = Root.CRoot
    module Atom = Root.Atom
    module Query = Root.Query
    module Update = Root.Update
    module Src = Src
    module Ext = Logcache.Ext

(* reasoning on descriptions, i.e., sets of features and descriptors *)

(*
    module Update =
      struct
	type t = Add of CRoot.t (* descriptor *) | Sub of CRoot.t | Seq of t * t | Nop

	let of_descr d =
	  List.fold_left (fun res x -> Seq (res, Add x)) Nop d

	let rec to_descr = function
	  | Add x -> [x]
	  | Sub x -> []
	  | Seq (u1,u2) -> to_descr u1 @ to_descr u2
	  | Nop -> []

	let rec of_query q =
	  match q with
	  | Query.Atom a ->
	      let f = Atom.formula a in
	      if CRoot.desc f
	      then Add f
	      else Nop
	  | Query.Not (Query.Atom a) ->
	      let f = Atom.formula a in
	      Sub f
	  | Query.Not (Query.Not q1) ->
              of_query q1
	  | Query.Not (Query.Or (q1,q2)) ->
	      of_query (Query.And (Query.Not q1, Query.Not q2))
	  | Query.And (q1,q2) ->
	      let u1 = of_query q1 in
	      let u2 = of_query q2 in
	      Seq (u1,u2)
	  | _ ->
	      Nop


	open Token

	let rec parse = parser
	  | [<u = parse_atom; ff = parse_suite>] -> ff u
	and parse_suite = parser
	  | [<'Ident "and"; u2 = parse>] -> (fun u1 -> Seq (u1,u2))
	  | [<>] -> (fun u -> u)
	and parse_atom = parser
	  | [<'Ident "all">] -> Nop
	  | [<'Ident "not"; f = CRoot.parse>] -> Sub f
	  | [<d = CRoot.parse>] -> if CRoot.desc d then Add d else raise (Stream.Error "Object.Update.parse: not a descriptor")

	let rec print = function
	  | Add x -> CRoot.print x
	  | Sub x -> Ident "not" :: PP_tilda :: CRoot.print x
	  | Seq (u1,u2) ->
	      ( match print u1, print u2 with
	      | [Ident "all"], toks2 -> toks2
	      | toks1, [Ident "all"] -> toks1
	      | toks1, toks2 -> toks1 @ PP_tilda :: Ident "and" :: PP_tilda :: toks2)
	  | Nop -> [Ident "all"]
      end
*)

    module Descr =
      struct
	type t = CRoot.t list

	let empty = []

	let entails : t -> CRoot.t -> bool =
	  fun ld d' ->
	    List.exists (fun d -> CRoot.entails d d') ld

	let rec weak_conj l1 = function
	  | [] -> l1
	  | x2::l2 -> weak_conj (weak_conj_unit x2 l1) l2
	and weak_conj_unit x = function
	  | [] -> [x]
	  | y::l ->
              if CRoot.entails x y then weak_conj_unit x l
              else if CRoot.entails y x then y::l
              else y::weak_conj_unit x l

	let rec add ld e =
	  let ld' =
	    Common.mapfilter
	      (fun d ->
		try Some (CRoot.add d e)
		with Not_found ->
		  if CRoot.entails e d
		  then None
		  else Some d)
	      ld in
	  if entails ld' e
	  then ld'
	  else e::ld
		    
	let sub ld e =
	  Common.mapfilter
	    (fun d ->
	      try Some (CRoot.sub d e)
	      with Not_found ->
		if CRoot.entails d e
		then None
		else Some d)
	    ld

	let replace ld e e'=
	  List.map
	    (fun d -> if d = e then e' else d)
	    ld

      (* returns the updated description, and the effective update *)
	let rec update ld = function
	  | Update.Add d ->
	      add ld d,
	      if entails ld d then Update.Nop else Update.Add d
	  | Update.Sub f ->
	      sub ld f,
	      List.fold_left
		(fun res d -> Update.Seq (Update.Sub d,res))
		Update.Nop
		(List.filter (fun d -> CRoot.entails d f) ld)
	  | Update.Replace (d,d') ->
	      replace ld d d',
	      if List.mem d ld then Update.Replace (d,d') else Update.Nop
	  | Update.Seq (u1,u2) ->
	      let ld1, u1' = update ld u1 in
	      let ld2, u2' = update ld1 u2 in
	      ld2, Update.Seq (u1',u2')
	  | Update.Nop ->
	      ld, Update.Nop

	let of_query q =
	  Update.to_descr (Update.of_query q)


	let rec parse : Syntax.t_stream -> t = parser
	  | [<'Token.Star>] -> empty
	  | [<d = CRoot.parse; ld = parse_suite>] -> if CRoot.desc d then d::ld else raise (Stream.Error "Object.Descr.parse: not a descriptor")
	and parse_suite = parser
	  | [<'Token.Comma; d = CRoot.parse; ld = parse_suite>] -> if CRoot.desc d then d::ld else raise (Stream.Error "Object.Descr.parse: not a descriptor")
	  | [<>] -> []
		
	let rec print : t -> Syntax.t_list = function
	  | [] -> [Token.Star]
	  | d::ld -> CRoot.print d @ print_suite ld
	and print_suite = function
	  | [] -> []
	  | d::ld -> Token.Comma :: CRoot.print d @ print_suite ld

      end


(* tbl_d associates all necessary information to objects *)

    type origin = External of Src.t * Src.ref | Internal of Source.preview * Source.action_arg list

    type t = {
	mutable origin : origin;    (* source and ref for external objects, title for internal objects *)
	mutable extr : Descr.t    (* extrinsic description *)
    }

    let tbl_d : (Logcache.oid,t) Hashtbl.t = Hashtbl.create 100
    let tbl_oid : (Src.t * Src.ref, Logcache.oid) Hashtbl.t = Hashtbl.create 100

(* get the origin of an object *)
    let get_origin oid =
      assert (!Root.cache # valid oid);
      (Hashtbl.find tbl_d oid).origin

(* get the extrinsic description of an object *)
    let get_extr oid =
      (Hashtbl.find tbl_d oid).extr

(* get the oid of an external object from its source and reference *)
    let get_oid src_ref =
      Hashtbl.find tbl_oid src_ref

(* make the description of an object from an intrinsic description *)
    let make_descr oid di =
      assert (!Root.cache # valid oid);
      di @ get_extr oid

(* get the description of an object *)
    let get_descr oid =
      assert (!Root.cache # valid oid);
      List.map (!Root.cache # formula) (!Root.cache # links oid)

(* add an object to the context, given its source, oid, intrinsic descr., and extrinsic descr.  *)
(* return add-updates *)
    let make (orig, di, de) =
      let oid = !Root.cache # alloc in
      Hashtbl.add tbl_d oid {origin = orig; extr = de};
      ( match orig with
      | External (src,ref) -> Hashtbl.add tbl_oid (src,ref) oid
      | Internal _ -> ());
      Root.Rules.get_updates oid
	(fun oid ->
	  match make_descr oid di with
	  | [] -> ignore (!Root.cache # link oid (!Root.cache # formula !Root.cache # top))
	  | l -> List.iter (fun f -> ignore (!Root.cache # link oid f)) l)

(* update an object from a new description *)
(* return rem-updates and add-updates *)
    let my_update (oid, d) =
      Root.Rules.get_updates oid
	(fun oid ->
	  !Root.cache # remove oid;
	  match d with
	  | [] -> ignore (!Root.cache # link oid (!Root.cache # formula !Root.cache # top))
	  | _ -> List.iter (fun f -> ignore (!Root.cache # link oid f)) d)

(* more efficient but BUGGY, don't know why
      let old_desc = LSet.of_list (List.map (!Root.cache # formula) (!Root.cache # links oid)) in
      let new_desc = LSet.of_list d in
      let l_link, l_unlink =
	LSet.fold
	  (fun (l1,l2) -> function
	    | LSet.Infst, x -> (l1, x::l2)
	    | LSet.Insnd, x -> (x::l1, l2)
	    | LSet.Inboth, _ -> (l1,l2))
	  ([],[])
	  old_desc new_desc in
      Root.Rules.get_updates oid
	(fun oid ->
	  List.iter (!Root.cache # unlink oid) l_unlink;
	  List.iter (fun f -> ignore (!Root.cache # link oid f)) l_link)
*)

(* update an object from a new intrinsic description *)
(* return rem-updates and add-updates *)
    let update (oid, d) =
      my_update (oid, make_descr oid d)

(* move an object by changing its description according to a formula *)
(* return rem-updates and add-updates *)
    let rec update_extr oid = function
      | Update.Add d -> ignore (!Root.cache # link oid d)
      | Update.Sub f -> !Root.cache # unlink oid f
      | Update.Replace (d,d') -> !Root.cache # replace oid d d'
      | Update.Seq (u1,u2) -> update_extr oid u1; update_extr oid u2
      | Update.Nop -> ()

    let move oid u =
      let de = get_extr oid in
      let de', u' = Descr.update de u in
      (Hashtbl.find tbl_d oid).extr <- de';
      Root.Rules.get_updates oid
	(fun oid -> update_extr oid u')

(* remove an object from the context *)
(* return rem-updates *)
    let remove oid =
      assert (!Root.cache # valid oid);
      let rem_up =
	Root.Rules.get_updates oid
	  (fun oid -> !Root.cache # remove oid) in
(*	    List.iter (!Root.cache # unlink_elt oid) (!Root.cache # links oid)) in *)
      ( match get_origin oid with
      | External (src,ref) -> Hashtbl.remove tbl_oid (src,ref)
      | Internal _ -> ());
      Hashtbl.remove tbl_d oid;
      !Root.cache # free oid;
      rem_up

(* move an object by changing its intrinsic description according to a formula *)
(* has a side effect on the source of oid *)
(* return rem-updates and add-updates *)

    let rec update_intr s r = function
      | Update.Add e ->
	  (try
	    let d = Src.add s r (CRoot.print e) in
	    Some (Descr.parse (Syntax.of_list d))
	  with _ -> None)
      | Update.Sub e ->
	  (try
	    let d = Src.sub s r (CRoot.print e) in
	    Some (Descr.parse (Syntax.of_list d))
	  with _ -> None)
      | Update.Replace (e,e') ->
	  (try
	    let d = Src.replace s r (CRoot.print e) (CRoot.print e') in
	    Some (Descr.parse (Syntax.of_list d))
	  with _ -> None)
      | Update.Seq (u1,u2) ->
	  let d1_opt = update_intr s r u1 in
	  let d2_opt = update_intr s r u2 in
	  if d2_opt=None then d1_opt else d2_opt
      | Update.Nop ->
	  None

    let move_intr oid u =
      match get_origin oid with
      | Internal _ -> []
      | External (s,r) ->
	  match update_intr s r u with
	  | None -> []
	  | Some di' ->
	      let d' = make_descr oid di' in
	      my_update (oid, d')


(* init, load, and save *)

    let init () =
      Root.init ();
      Hashtbl.clear tbl_d;
      Hashtbl.clear tbl_oid

    type data = Root.data * (Logcache.oid,t) Hashtbl.t * (Src.t * Src.ref, Logcache.oid) Hashtbl.t

    let save () = (Root.save (), tbl_d, tbl_oid)

    let load (r, d, o) =
      Root.load r;
      Hashtbl.clear tbl_d;
      Hashtbl.iter
	(fun oid x -> Hashtbl.add tbl_d oid x)
	d;
      Hashtbl.clear tbl_oid;
      Hashtbl.iter
	(fun url_ref oid -> Hashtbl.add tbl_oid url_ref oid)
	o

  end
