(** Source: BibTeX bibliography files. *)

open Source

module Param =
  struct
    let filter path = Filename.check_suffix path ".bib"

    let may_have_changed_since = Url.may_have_changed_since

    let text_of_source url = Url.path_of_url url

    let wrapper url objs =
      let main_pre = Url.preview_of_url url in
      let main_arg = {mime=Url.mime url "text/bibtex"; role="BibTeX"; location=url} in
      let main_obj = ("", ([], String main_pre, [])) in
      let lf_src = Url.fields_of_url "source" [] url in
      List.map
	(fun (ref, (lf, pre, args)) ->
	  (ref, (lf_src @ lf, pre, main_arg :: args)))
	(main_obj :: objs)
  end
module Make : T =
  Setobj_lexyacc.Make(Param)(Bibtex_p)(Bibtex_l)


(*
open Source

module Param =
  struct
    let text_of_source (path : string) = path
  end
module Make : T = Fileparts_lexyacc.Make(Param)(Bibtex_p)(Bibtex_l)(
  struct
    let suffix = ["bib"]
    let file_attributes = []
    let preview path ref pre = String pre
    let main_arg path = {mime="text/bibtex"; role="BibTeX"; location=path}
  end)
*)
