%{
let ret : string -> string = fun s -> print_string s; s

let get_attr attr lf =
  match List.assoc attr lf with
  | Fields.StringList l -> String.concat ", " l
  | Fields.Integer n -> string_of_int n
  | _ -> ""

let token_start = Start
%}

%token Start
%token Comma
%token LeftAcc
%token RightAcc
%token Quote
%token Eq
%token White
%token <string> Ident
%token <string> Others

%start start
%type <string * (Fields.t * Source.preview * Source.action_arg list)> start
%%


start:
  bibtype LeftAcc reference whites field fields { 
   let ref = $3 in
   let lf = ("type",Fields.StringList [String.lowercase $1]) :: ("ref",Fields.StringList [$3]) :: $5 @ $6 in
   let key = try get_attr "key" lf with _ -> "" in
   let authors = try get_attr "authors" lf with _ -> "" in
   let title = try get_attr "title" lf ^ ". " with _ -> "" in
   let where = try get_attr "journal" lf ^ ", " with _ -> try get_attr "booktitle" lf ^ ", " with _ -> "" in
   let year = try get_attr "year" lf ^ "." with _ -> "" in
   let pre = "[" ^ ref ^ "] " ^ authors ^ "\n" ^ title ^ "\n" ^ where ^ year in
   let ref_key = ref^"_"^key in
   let ref_arg = Some ({Source.mime="ref"; Source.role="Reference"; Source.location=ref_key}) in
   let pattern = Some ({Source.mime="pattern"; Source.role=ref; Source.location="{" ^ ref ^ ","}) in
   let url = try Some ({Source.mime="url/text/html"; Source.role="Web page"; Source.location=get_attr "url" lf}) with _ -> None in
   let pdf = try Some ({Source.mime="url/application/pdf"; Source.role="PDF"; Source.location=get_attr "pdf" lf}) with _ -> None in
   let ee = try Some ({Source.mime="url/text/html"; Source.role="Electronic Edition"; Source.location=get_attr "ee" lf}) with _ -> None in
   let action_args = Common.mapfilter (fun x -> x) [ref_arg; pattern; url; pdf; ee] in
   ref_key, (lf, Source.String pre, action_args) }

bibtype:
| Ident { $1 }

reference:
| Comma { "" }
| White reference { $2 }
| Ident reference { $1^$2 }
| Others reference { $1^$2 }

fields:
| Comma whites field fields { List.fold_right Fields.add $3 $4 }
| Comma whites RightAcc { Fields.init }
| RightAcc { Fields.init }

field:
| Ident whites Eq whites content {
  let attr = String.lowercase $1 in
  match attr with
  | "year"
  | "volume"
  | "number" ->
      [attr, (try Fields.Integer (int_of_string $5) with _ -> Fields.None)]
  | "author" ->
      let authors = Str.split (Str.regexp " +and +") $5 in
      let simpl_firstname fn =
	Str.global_replace (Str.regexp "\\b\\([^\\. \t-]\\)[^\\. \t-]*[^\\. \t-]\\b") "\\1." fn in
      let authors' = List.map (fun s ->
	match Str.split (Str.regexp " *, *") s with
	| lastname::firstname::_ ->
	    lastname ^ ", " ^ simpl_firstname firstname
	| _ ->
	    try
	      let i = String.rindex s ' ' in
	      let lastname, firstname = String.sub s (i+1) (String.length s - i - 1), String.sub s 0 i in
	      lastname ^ ", " ^ simpl_firstname firstname
	    with Not_found -> s) authors in
      ("authors", Fields.StringList [String.concat " and " authors']) :: List.map (fun a -> attr, Fields.StringList [a]) authors'
  | "keywords" ->
      let keywords = Str.split (Str.regexp " *[,;] *") $5 in
      List.map (fun k -> "keyword", Fields.StringList [String.lowercase k]) keywords
  | "url"
  | "ps"
  | "pdf"
  | "ee" ->
      Url.fields_of_url attr [] $5
  | _ ->
      [attr, Fields.StringList [$5]] }

content:
| Quote to_quote to_end { $2 }
| LeftAcc to_rightacc to_end { $2 }
| Ident to_end { $1^$2 }
| Others to_end { $1^$2 }

to_end:
| Start to_end { "@"^$2 }
| LeftAcc to_rightacc to_end { "{"^$2^"}"^$3 }
| Quote to_quote to_end { "\""^$2^"\""^$3 }
| Eq to_end { "="^$2 }
| White to_end { if $2 = "" then "" else " "^$2 }
| Ident to_end { $1^$2 }
| Others to_end { $1^$2 }
|  { "" }

whites:
| White {}
|   {}

to_rightacc:
| LeftAcc to_rightacc to_rightacc { "{"^$2^"}"^$3 }
| RightAcc { "" }
| Quote to_quote to_rightacc { "\""^$2^"\""^$3 }
| Start to_rightacc { "@"^$2 }
| Comma to_rightacc { ","^$2 }
| Eq to_rightacc { "="^$2 }
| White to_rightacc { " "^$2 }
| Ident to_rightacc { $1^$2 }
| Others to_rightacc { $1^$2 }

to_quote:
| LeftAcc to_rightacc to_quote { "{"^$2^"}"^$3 }
| Quote { "" }
| Start to_quote { "@"^$2 }
| Comma to_quote { ","^$2 }
| Eq to_quote { "="^$2 }
| White to_quote { " "^$2 }
| Ident to_quote { $1^$2 }
| Others to_quote { $1^$2 }


%%

