(** Source: Mozilla or any html bookmark files. *)

open Source

module Make =
  Setobj.Make
    (struct
      let filter path = Filename.basename path = "bookmarks.html"

      let may_have_changed_since = Url.may_have_changed_since

      let re_href = "HREF=\"\\([^\"]+\\)\"[^>]*>\\([^<]+\\)<"
      let re_h3_dl = "<H3[^>]*>\\([^<]+\\)<"
      let re_findl = "\\(</DL>\\)"

      let get_objs url proper_parts =
	let s = Common.string_of_file (Url.path_of_url url) in
	let s = Str.global_replace (Str.regexp "\n") "" s in (* line returns are irrelevant in HTML *)
	let extr = ref [] in
	let l = ref [] in
	let lf_src = Url.fields_of_url "source" [] url in
	let arg_src = {mime=Url.mime url "text/html"; role="Bookmarks"; location=url} in
	let start = ref 0 in
	if proper_parts then
	  (try while true do
	    ignore (Str.search_forward (Str.regexp (re_href ^ "\\|" ^ re_h3_dl ^ "\\|" ^ re_findl)) s !start);
	    let url_opt = try Some (Str.matched_group 1 s) with Not_found -> None in
	    let title_opt = try Some (Str.matched_group 2 s) with Not_found -> None in
	    let p_opt = try Some (Str.matched_group 3 s) with Not_found -> None in
	    let fin_opt = try Some (Str.matched_group 4 s) with Not_found -> None in
	    start := Str.match_end ();
	    ( match url_opt, title_opt with
	    | Some url, Some title ->
		let lf =
		  ("source", Fields.Composed ("part",Fields.None))::
		  Url.fields_of_url "url" [] url @
		  ("title",Fields.StringList [title])::
		  List.map (fun t -> ("",Fields.TermAsAttr t)) !extr in
		let action_args = [{mime="url/text/html"; role=""; location=url}; arg_src] in
		l := (url, (lf_src @ lf, Source.String title, action_args)) :: !l
	    | _, _ -> ());
	    ( match p_opt with
	    | Some p -> 
		extr := p::!extr
	    | _ -> ());
	    ( match fin_opt with
	    | Some _ -> 
		extr := List.tl !extr
	    | _ -> ())
	done with _ -> ());
	let file_pre = Url.preview_of_url url in
	("",(lf_src, String file_pre, arg_src :: Url.args_of_url url))::List.rev !l
    end)

(*
module Make : Source.T = Fileparts.Make(Transducer)(
  struct
    let suffix = ["html";"htm"]
    let file_attributes = []
    let preview path ref pre = Source.String pre
    let main_arg path = {mime="text/html"; role="Bookmarks"; location=path}
  end)
*)
