(** Source: DBLP search results. *)

let find_all_matched regexp s =
  let _, occs =
    Common.fold_while
      (fun (pos,res) ->
	try
	  ignore (Str.search_forward regexp s pos);
	  let pos' = Str.match_end () in
	  let occ = Str.matched_group 1 s in
	  Some (pos',occ::res)
	with Not_found -> None)
      (0,[]) in
  List.rev occs

let re1 = Str.regexp "\\[DBLP:\\([a-zA-Z0-9-/]+\\)"
let re2 = Str.regexp "/rec/bibtex/\\([^\"]+\\)"

open Source

module Param =
  struct
    let filter =
      let re = Str.regexp_string "http://www.informatik.uni-trier.de/" in
      fun url -> Str.string_match re url 0

    let may_have_changed_since = Url.may_have_changed_since

    let get_srcs url recursive f =
      let html = Common.string_of_file (Url.path_of_url url) in
      let refs1 = find_all_matched re1 html in
      let refs2 = find_all_matched re2 html in
      List.map
	(fun ref -> prerr_endline ref; "http://dblp.uni-trier.de/rec/bibtex/" ^ ref)
	(refs1 @ refs2)

    let move_src _ src = src
  end
module Make : Source.T =
  Setsrc.Make(Param)

(*
module Param =
  struct
    let text_of_source (path : string) = bibtex_file path
    let whole_as_object = true
  end
module Make : Source.T = Urlparts_lexyacc.Make(Param)(Bibtex_p)(Bibtex_l)(
  struct
    let re = Str.regexp_string "http://www.informatik.uni-trier.de/"
    let filter url = Str.string_match re url 0
    let url_attributes = []
    let preview url ref pre = Source.String pre
    let main_arg url = {mime="url/text/html"; role="DBLP results page"; location=url}
  end)
*)
