(** Source: Esperanto sentences from .eo files. *)

open Source

module Param =
  struct
    let filter url = Filename.check_suffix url ".eo"

    let may_have_changed_since = Url.may_have_changed_since

    let read_text filename =
      try
	let ch = open_in filename in
	let buf = Buffer.create 10000 in
	begin try while true do
	  Buffer.add_string buf (input_line ch ^ " ")
	done with _ -> () end;
	close_in ch;
	Buffer.contents buf
      with _ -> prerr_endline (filename ^ " cannot be read"); ""

    let get_objs url proper_parts =
      let lf_src = Url.fields_of_url "source" [] url in
      let src_arg = {mime=Url.mime url "text"; role=""; location=url} in
      let _, objs =
	if proper_parts
	then
	  let basename = Filename.basename url in
	  let docname = try Filename.chop_extension basename with _ -> basename in
	  let text = read_text (Url.path_of_url url) in
	  let text = Str.global_replace (Str.regexp "[ \t][ \t]+") " " text in
	  let text = Str.global_replace (Str.regexp "t\\.e\\. ") "t.e., " text in
	  let text = Str.global_replace (Str.regexp "k\\.s\\. ") "k.s., " text in
	  let text = Str.global_replace (Str.regexp "ekz\\. ") "ekz., " text in
	  let text = Str.global_replace (Str.regexp "\\([.?!]+\\)[ \t]+") "\\1\n" text in
	  let sentences = Str.split (Str.regexp "\n") text in
	  List.fold_left
	    (fun (i,objs) s ->
	      let words =
		List.map String.lowercase 
		  (List.filter (fun w -> String.length w > 1 && not (String.contains w '-'))
		     (Str.split (Str.regexp "[^a-zA-Z']")
			s)) in
	      if words = []
	      then
		(i,objs)
	      else
		let ref = string_of_int (i+1) in
		let pre = s ^ " [" ^ docname ^ "]" in
		let lf = ("doc",Fields.StringList [docname])::List.map (fun w -> (w,Fields.None)) words in
		let action_args = [] in
		let obj = ref, (lf_src @ lf, String pre, src_arg::action_args) in
		(i+1,obj::objs))
	    (0,[])
	    sentences
	else (0,[]) in
      let src_pre = url in
      ("",(lf_src, String src_pre, [src_arg]))::objs
  end

module Make : T = Setobj.Make(Param)

(*
module Make : Source.T = Fileparts.Make(Transducer)(
  struct
    let suffix = ["eo"]
    let file_attributes = []
    open Token
    let content_of_path_ref path ref =
      [Ident "file"; PP_tilda; Minus; Ident "k"; PP_tilda; String ref; PP_tilda; String path]
    let preview path ref pre = Source.String pre
    let main_arg path = {mime="text"; role=""; location=path}
  end)
*)
