(** Source: Java source files. *)

open Source

module Param =
  struct
    let filter url = Filename.check_suffix url ".java"

    let may_have_changed_since = Url.may_have_changed_since

    let rec parse_axiom = parser
      | [<'Token.Ident "axiom"; f = non_comma; 'Token.Comma; g = non_comma>] -> (f,g)
    and non_comma = parser
      | [<'tok when tok <> Token.Comma; toks = non_comma>] -> tok::toks
      | [<>] -> []

    let parse_mk = parser
      | [<'Token.Ident "mk"; 'Token.String ref; 'Token.String desc; str>] ->
	  (ref,desc,Syntax.stringizer (Syntax.list_of_stream str))

    let get_objs url proper_parts =
      let lf_src = Url.fields_of_url "source" [] url in
      let src_arg = {mime=Url.mime url "text/java"; role="Java source"; location=url} in
      let parts = 
	if proper_parts
	then begin
	  let temp_file = Filename.temp_file "camelisdoc" "" in
	  ignore (Sys.command ("javadoc -doclet CamelisDoc -scriptprefix " ^ temp_file ^ " -private " ^ (Url.path_of_url url)));
	  let ch_in = open_in temp_file in
	  Common.fold_in_channel
	    (fun res line ->
	      match Syntax.from_string line with parser
	      | [<(f,g) = parse_axiom>] -> res
	      | [<(ref,desc,javatype) = parse_mk>] ->
		  let pre = ref ^ " : " ^ desc in
		  let lf = [("source", Fields.Composed ("part",Fields.None)); ("desc",Fields.StringList [desc]); ("",Fields.Verbatim javatype)] in
		  let action_args = {mime="pattern"; role=ref; location=ref}::src_arg::Url.args_of_url url in
		  (ref,(lf_src @ lf, String pre, action_args))::res)
	    []
	    ch_in end
	else [] in
      let src_pre = Url.preview_of_url url in
      ("",(lf_src, String src_pre, [src_arg]))::parts
  end

module Make : Source.T =
  Setobj.Make(Param)

(*
module Make : Source.T = Fileparts.Make(Transducer)
    (struct
      let suffix = ["java"]
      let file_attributes = []
      let preview path ref pre = Source.String pre
      let main_arg path = {mime="text/java"; role="Java source"; location=path}
    end)
*)
