(** Source: ML interface files (.mli). *)

open Source

module Param =
  struct
    let filter url = Filename.check_suffix url ".mli"

    let may_have_changed_since = Url.may_have_changed_since

    let re_val = "[ \t\n]val[ ]+\\([^ :]+\\)[ ]*\\(:[^;\n]+\\)"
    let re_module = "[ \t\n]module[ ]+\\([A-Z][A-Za-z0-9_']+\\)"
    let re_end = "\\([ \t\n]end[ \t\n]\\)"

    let get_objs url proper_parts =
      let l = ref [] in
      let lf_src = Url.fields_of_url "source" [] url in
      let src_arg = {mime=Url.mime url "text/mli"; role="ML interface"; location=url} in
      if proper_parts then begin
	let s = Common.string_of_file (Url.path_of_url url) in
	let s = Str.global_replace (Str.regexp "(\\*+[^*]+\\*)") "" s in (* removing comments *)
	let main_module =
	  let m = Filename.chop_extension (Filename.basename url) in
	  m.[0] <- Char.uppercase m.[0];
	  m in
	let extr = ref [] in
	let start = ref 0 in
	(try while true do
	  ignore (Str.search_forward (Str.regexp (re_val ^ "\\|" ^ re_module ^ "\\|" ^ re_end)) s !start);
	  let name_opt = try Some (Str.matched_group 1 s) with Not_found -> None in
	  let mltype_opt = try Some (Str.matched_group 2 s) with Not_found -> None in
	  let module_opt = try Some (Str.matched_group 3 s) with Not_found -> None in
	  let end_opt = try Some (Str.matched_group 4 s) with Not_found -> None in
	  start := Str.match_end ();
	  ( match name_opt, mltype_opt with
	  | Some name, Some mltype ->
(* print_string name; print_endline mltype; *)
	      let ref = main_module ^ "." ^ List.fold_left (fun res m -> m ^ "." ^ res) name !extr in
	      let pre = ref ^ " " ^ mltype in
	      let lf =
		("source", Fields.Composed ("part",Fields.None))::
		("name",Fields.StringList [name])::
		("mltype",Fields.Verbatim (Str.global_replace (Str.regexp "'") "`" mltype))::
		List.map (fun m -> ("module",Fields.StringList [m])) (main_module::!extr) in
	      let action_args = {mime="pattern"; role=name; location="val " ^ name}::src_arg::Url.args_of_url url in
	      l := (ref,(lf_src @ lf, String pre, action_args)) :: !l
	  | _, _ -> ());
	  ( match module_opt with
	  | Some m ->
(* print_string "module "; print_endline m; *)
	      extr := m::!extr
	  | _ -> ());
	  ( match end_opt with
	  | Some _ -> 
(* print_endline "end"; *)
	      (try extr := List.tl !extr with _ -> ())
	  | _ -> ())
	done with _ -> ()) end;
      let src_pre = Url.preview_of_url url in
      ("",(lf_src, String src_pre, [src_arg]))::List.rev !l
  end

module Make : Source.T =
  Setobj.Make(Param)

(*
module Make : Source.T = Fileparts.Make(Transducer)
    (struct
      let suffix = ["mli"]
      let file_attributes = []
      let preview path ref pre = Source.String pre
      let main_arg path = {mime="text/mli"; role="ML interface"; location=path}
    end)
*)
