(** Source: tags from MP3 audio files. *)

module Make : Source.T =
  Sglobj.Make
    (struct
      let filter path = Filename.check_suffix path ".mp3"

      let may_have_changed_since = Url.may_have_changed_since

      let get_obj url =
	let path = Url.path_of_url url in
	let tags =
	  try Mp3id3.Id3Local.read_tag path
	  with _ -> [] in
	let get_val a =
	  try Some (List.assoc a tags)
	  with _ -> None in
	let get_stringlist a =
	  try Some (String.lowercase a, Fields.StringList [List.assoc a tags])
	  with _ -> None in
	let get_nat a =
	  try Some (String.lowercase a, Fields.Integer (int_of_string (List.assoc a tags)))
	  with _ -> None in
	let artist = get_val "ARTIST" in
	let title = get_val "TITLE" in
	let lf =
	  Url.fields_of_url "source" [] url @
	  (Common.mapfilter get_stringlist ["ARTIST"; "ALBUM"; "TITLE"; "GENRE"]) @
	  (Common.mapfilter get_nat ["DURATION"; "DATE"; "TRACKNUMBER"]) in
	let pre =
	  match artist, title with
	  | None, None -> Filename.basename (Filename.chop_extension path)
	  | Some a, None -> a ^ " - " ^ Filename.basename (Filename.chop_extension path)
	  | None, Some t -> t
	  | Some a, Some t -> a ^ " - " ^ t in
	let lf =
	  if lf = []
	  then
	    let album = Filename.basename (Filename.dirname path) in
	    let title = Filename.basename (Filename.chop_extension path) in
	    [("album",Fields.StringList [album]);
	     ("title",Fields.StringList [title])]
	  else lf in
	let action_args = {Source.mime=Url.mime url "audio/mp3"; Source.role=""; Source.location=url}::Url.args_of_url url in
	lf, Source.String pre, action_args
    end)
