%{

let int_of_string s =
  try int_of_string s
  with _ -> 0

let rec get_attr attr = function
  | [] -> raise Not_found
  | (_, Fields.Date (d,m,y))::_ when attr = "date" -> String.concat " " [string_of_int d; m; string_of_int y]
  | (a,v)::l ->
      if a = attr
      then
	match v with
	| Fields.StringList l -> String.concat ", " l
	| Fields.Integer n -> string_of_int n
	| _ -> ""
      else
	get_attr attr l

(*
let get_attr attr lf =
  match List.assoc attr lf with
  | Fields.StringList l -> String.concat ", " l
  | Fields.Integer n -> string_of_int n
  | Fields.Date (d,m,y) -> String.concat " " [string_of_int d; m; string_of_int y]
  | _ -> ""
*)

%}

%token Start
%token <string> IDColon
%token <string> DateColon
%token <string> FromColon
%token <string> ToColon
%token <string> CcColon
%token <string> SubjectColon
%token <string> LengthColon
%token <string> Sep
%token <string> Integer
%token <string> Word
%token <string> Whites
%token <string> Others
%token EOL

%start start
%type <(int * string * int) * (int * int * int)> date
%type <string * (Fields.t * Source.preview * Source.action_arg list)> start
%%

start:
  endline fields  {
    let ref, lf = $2 in
    let date = try get_attr "date" lf with _ -> "?" in
    let from = try get_attr "from" lf with _ -> "?" in
    let subj = try get_attr "subject" lf with _ -> "?" in
    let pre = subj ^ ", " ^ from ^ ", " ^ date in
    let action_args = [{Source.mime="pattern"; Source.role=""; Source.location=ref}] in
    ref, (lf, Source.String pre, action_args)
}

fields:
| field fields {
    match $1 with
      ["id", Fields.StringList (x::l)] -> let ref, lf = $2 in x, lf
    | lf' -> let ref, lf = $2 in ref, List.fold_right (fun f res -> Fields.add f res) lf' lf }
| EOL  { "", Fields.init }

field:
| IDColon endline { ["id", Fields.StringList [$2]] }
| DateColon date { let (d,m,y), (h,mn,s) = $2 in ["", Fields.Date (d,m,y); "", Fields.Time (h,mn,s)] }
| FromColon adresses { ["from", Fields.StringList $2] }
| ToColon adresses { ["to", Fields.StringList $2] }
| CcColon adresses { ["cc", Fields.StringList $2] }
| SubjectColon subject { ["subject", Fields.StringList [$2]] }
| LengthColon Integer endline { ["length", Fields.Integer (int_of_string $2)] }
| Word endline { [] }

date:
  Word Sep Whites Integer Whites Word Whites Integer Whites Integer Sep Integer Sep Integer endline { (int_of_string $4, $6, int_of_string $8), (int_of_string $10, int_of_string $12, int_of_string $14) }

adresses:
  adress Sep adresses  { match $1 with None -> $3 | Some s -> s::$3 }
| adress EOL  { match $1 with None -> [] | Some s -> [s] }

adress:
| adress not_word { $1 }
| adress Word  { Some $2 }
| adress EOL Whites  { $1 }
| { None }

subject:
  something subject  { $1^$2 }
| EOL Whites subject { $3 }
| EOL { "" }

/*
endline:
| something endline  { $1^$2 }
| EOL Whites endline { $3 }
| EOL { "" }
*/

endline:
| endline2 EOL  { $1 }

endline2:
| endline2 something  { $1 ^ $2 }
| endline2 EOL Whites { $1 ^ " " }
| something           { $1 }

something:
| Sep { $1 }
| Word { $1 }
| not_word { $1 }

not_word:
| IDColon { $1 }
| DateColon { $1 }
| FromColon { $1 }
| ToColon { $1 }
| CcColon { $1 }
| SubjectColon { $1 }
| LengthColon  { $1 }
| Integer { $1 }
| Whites { $1 }
| Others { $1 }


%%

let token_start = Start
