(** Source: http://www.oiseaux.net/ *)

let untag s = Str.global_replace (Str.regexp "</?[a-z]+>") "" s

let clean s =
  let s1 = Str.global_replace (Str.regexp "[ \n\t]+") " " s in
  let s2 = Str.global_replace (Str.regexp "\\(^ \\| $\\)") "" s1 in
  s2

let get re s =
  let pos = Str.search_forward (Str.regexp re) s 0 in
  Str.matched_group 1 s

let get_re a re s =
  try
    let pos = Str.search_forward (Str.regexp re) s 0 in
    [ (a, Fields.StringList [clean (Str.matched_group 1 s)]) ]
  with Not_found -> []

let get_re2 a b re s =
  try
    let pos = Str.search_forward (Str.regexp re) s 0 in
    [ (a, Fields.Composed (b, Fields.StringList [clean (Str.matched_group 1 s)])) ]
  with Not_found -> []

let get_interval a re s =
  let re' = re ^ "\\([0-9]+\\)\\([^0-9<]+\\([0-9]+\\)\\)?" in
  try
    let pos = Str.search_forward (Str.regexp re') s 0 in
    let inf = int_of_string (Str.matched_group 1 s) in
    let sup =
      try int_of_string (Str.matched_group 3 s)
      with _ -> inf in
    [ (a, Fields.Composed ("min", Fields.Integer inf));
      (a, Fields.Composed ("max", Fields.Integer sup)) ]
  with Not_found -> []


open Source

module Param =
  struct
    let filter =
      let re = Str.regexp_string "http://www.oiseaux.net/oiseaux/" in
      fun url -> Str.string_match re url 0

    let may_have_changed_since url t = true

    let get_obj url =
      let path = Filename.concat (Sys.getcwd ()) (Filename.basename url) in
      if not (Sys.file_exists path) then Url.wget url path;
      let html = Common.string_of_file path in
      let text = Str.global_replace (Str.regexp "<i> Son extrait[^<]*\\(</i>\\|<[^<]*\\)*\n") "" html in
      let text = Str.global_replace (Str.regexp "<br>") ">" text in
      let text = Str.global_replace (Str.regexp "<[^>]+>") "" text in
      let nom = get "<h1>\n?\\([^<]+\\)<" html in
      let lf =
	get_re2 "nom" "commun" "<h1>\n?\\([^<]+\\)<" html @
	get_re "ordre" "Ordre</span> :[ \n\t]*\\([^<]+\\)<" html @
	get_re "famille" "Famille</span> :[^>]+>[^>]+>\\([^<]+\\)<" html @
	get_re2 "nom" "scientifique" "<i>\\([^<]+\\)<" html @
	get_interval "taille" "Taille :[ ]+" html @
	get_interval "envergure" "Envergure :[ ]+" html @
	get_interval "poids" "Poids :[ ]+" html @
	get_interval "longevite" "Longévité</span>[ ]*:[ ]+" html @
(*	get_re "distribution" ">Distribution :[ \n\t]+\\([^>]+\\)" text @ *)
(*	get_re "statut" ">Statut :[ \n\t]+\\([^\n]+\\)" text @ *)
(*
	get_re "identification" ">Identification :[^<]+</span><br>\\([^<]+\\)<" text @
	get_re "chant" ">Chant :[ \n\t>]+\\([^>]+\\)" text @
	get_re "habitat" ">Habitat :[ \n\t]+\\([^>]+\\)" text @
	get_re "comportements" ">Comportements :[ \n\t]+\\([^>]+\\)" text @
	get_re "vol" ">Vol :[ \n\t]+\\([^>]+\\)" text @
	get_re "nidification" ">Nidification :[ \n\t]+\\([^>]+\\)" text @
	get_re "regime" ">Rgime :[ \n\t]+\\([^>]+\\)" text @
*)
	[] in
      let path_img_opt =
	try
	  let url_img_rel = get "<img src=\"\\([^\"]+\\.jpg\\)\"" html in
	  let path_img = Filename.concat (Sys.getcwd ()) (Str.global_replace (Str.regexp "/") "_" url_img_rel) in
	  let url_img = Filename.dirname url ^ "/" ^ url_img_rel in
	  let code_img =
	    if not (Sys.file_exists path_img)
	    then Sys.command ("wget -q -O " ^ path_img ^ " \"" ^ url_img ^ "\"")
	    else 0 in
	  if code_img = 0
	  then Some path_img
	  else failwith ""
	with _ -> None in
      let path_mp3_opt =
	try
	  let url_mp3_rel = get "son=\\.\\./\\([^\"]+\\.mp3\\)\"" html in
	  let path_mp3 = Filename.concat (Sys.getcwd ()) (Str.global_replace (Str.regexp "/") "_" url_mp3_rel) in
	  let url_mp3 = "http://www.oiseaux.net/" ^ url_mp3_rel in
	  let code_mp3 =
	    if not (Sys.file_exists path_mp3)
	    then Sys.command ("wget -q -O " ^ path_mp3 ^ " \"" ^ url_mp3 ^ "\"")
	    else 0 in
	  if code_mp3 = 0
	  then Some path_mp3
	  else failwith ""
	with _ -> None in
      let pre = Common.fold_option (fun path_img -> Picture (path_img, nom)) (String nom) path_img_opt in
      let args =
	[ { mime = "url/text/html"; role = "Fiche web"; location = url} ] @
	( Common.fold_option (fun path_img -> [{ mime = "image/jpeg"; role = "Image"; location = path_img}]) [] path_img_opt ) @
	( Common.fold_option (fun path_mp3 -> [{ mime = "audio/mp3"; role = "Chant"; location = path_mp3}]) [] path_mp3_opt ) in
      (lf, pre, args)

  end
module Make : T =
  Sglobj.Make(Param)
