
module Make
(Param : sig
  val filter : Source.url -> bool
  val may_have_changed_since : Source.url -> float -> bool
  val text_of_source : Source.url -> Source.url (* result is a file path *)
  val wrapper : Source.url -> (string * (Fields.t * Source.preview * Source.action_arg list)) list -> (string * (Fields.t * Source.preview * Source.action_arg list)) list
end)
(Parser : sig
  type token
  val token_start : token
  val start : (Lexing.lexbuf -> token) -> Lexing.lexbuf -> string * (Fields.t * Source.preview * Source.action_arg list)
end)
(Lexer : sig
  exception Eof
  val token : Lexing.lexbuf -> Parser.token
end) =
  Setobj.Make
    (struct
      let filter = Param.filter

      let may_have_changed_since = Param.may_have_changed_since

      let get_objs url proper_parts =
	let rec parse lexbuf =
	  try
	    while Lexer.token lexbuf <> Parser.token_start do () done;
	    let ref, (lf, pre, args) = Parser.start Lexer.token lexbuf in
	    (ref, (("source", Fields.Composed ("part", Fields.None))::lf, pre, args)) :: parse lexbuf
	  with
	  | Parsing.Parse_error -> parse lexbuf
	  | Lexer.Eof -> [] in

	let text = Param.text_of_source url in
	let parts =
	  if proper_parts
	  then
	    try
	      let lexbuf = Lexing.from_channel (open_in text) in
	      parse lexbuf
	    with e ->
	      print_endline (Printexc.to_string e);
	      []
	  else [] in
	Param.wrapper url parts
    end)
